#!/usr/bin/env python3

import argparse
import os
import sys

import pyperclip

from emgen import __version__
from emgen.core import localpart

DISPLAY = os.getenv("DISPLAY")
LINUX = os.name == "posix"


def _has_clipboard():
    try:
        original = pyperclip.paste()
        pyperclip.copy(original)
        return True
    except pyperclip.PyperclipException:
        return False


def main():
    has_clipboard = _has_clipboard()

    parser = argparse.ArgumentParser(
        prog="emgen", description="Generate random email addresses.",
    )
    parser.add_argument(
        "-c",
        "--clipboard",
        action="store_true",
        default=False,
        help="copy addr to clipboard (default: %(default)s)"
        if has_clipboard
        else argparse.SUPPRESS,
    )
    parser.add_argument(
        "-d",
        "--domain",
        default="example.com",
        type=str,
        help="set the addr domain portion (default: %(default)s)",
        metavar="DOMAIN",
    )
    parser.add_argument(
        "-l",
        "--length",
        default=8,
        type=int,
        help="length of the local-part (from 1 to 64) (default: %(default)s)",
        metavar="LENGTH",
    )
    parser.add_argument(
        "-v", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    args = parser.parse_args()

    lp = localpart(args.length)
    addr = f"{lp}@{args.domain}"

    if args.clipboard:
        if has_clipboard:
            pyperclip.copy(addr)
        else:
            warning = "warning: could not find a clipboard for your system"
            print(warning, file=sys.stderr)

    print(addr)



# -*- coding:utf-8 -*-

from string import ascii_lowercase, digits

from emgen.core import localpart

CHARS = digits + ascii_lowercase


def test_localpart(trials: int = 99_999, length: int = 8) -> None:
    """Test generation of random email address local-parts.

    Args:
        trials (int, optional): Number of tests to perform. Defaults to 99_999.
        length (int, optional): Length of the local-part (from 1 to 64).
            Defaults to 8.

    """
    for _ in range(trials):
        lp = localpart(length=length)
        len_lp = len(lp)

        assert len_lp == length
        assert 1 <= len_lp <= 64
        assert set(lp) <= set(CHARS)



# -*- coding:utf-8 -*-

import random
from string import ascii_lowercase, digits

CHARS = ascii_lowercase + digits


def localpart(length: int = 8) -> str:
    """Generate a random email address local-part.

    Local-parts are generated using a minimal set of the characters specified
    in RFC 5322.

    Args:
        length (int, optional): Length of the local-part (from 1 to 64).
            Defaults to 8.

    Returns:
        str: A randomly generated email address local-part.

    See Also:
        https://tools.ietf.org/html/rfc5322#section-3.4.1
        https://www.jochentopf.com/email/address.html

    """
    if not 1 <= length <= 64:
        raise ValueError(f"invalid value for `length`: {length}")
    r = random.SystemRandom()
    lp = "".join(r.choices(CHARS, k=length))
    return lp
